<?php

/**
 * @file robotagger.admin.inc.
 */

function robotagger_settings_page() {
  $output = '';
  $form = drupal_get_form('robotagger_settings_form');
  $output .= drupal_render($form);
  return $output;
}

function robotagger_settings_form($form, $form_state) {
  $form['#attached']['js'] = array(drupal_get_path('module', 'robotagger') . '/js/robotagger.js');
  $form['robotagger_global_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Global settings'),
    '#description' => t('Please note. You need to add the right taxonomy fields for each vocabularies in each content type, where you want to use it.'),
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
  );
  $form['robotagger_global_wrapper']['robotagger_global_sendrequest'] = array(
    '#type' => 'select',
    '#title' => t('Send request'),
    '#options' => array('disabled' => t('disabled'), 'auto' => t('automatic'), 'manual' => t('manual')),
    '#default_value' => variable_get('robotagger_global_sendrequest', 'auto'),
    '#description' => t('automatic - After submitting the content, request the RoboTagger-Webservice'),
  );
  $form['robotagger_global_wrapper']['robotagger_global_confirmtags'] = array(
    '#type' => 'select',
    '#title' => t('Confirm returned tags'),
    '#options' => array('auto' => t('automatic'), 'manual' => t('manual')),
    '#default_value' => variable_get('robotagger_global_confirmtags', 'auto'),
    '#description' => t('manual - Before the tags are saved, you can confirm or edit them. automatic - the module save the tags immediately'),
  );
  $form['robotagger_global_wrapper']['robotagger_global_caching'] = array(
    '#type' => 'select',
    '#title' => t('Caching returned data'),
    '#options' => array(TRUE => t('yes'), FALSE => t('no')),
    '#default_value' => variable_get('robotagger_global_caching', TRUE),
    '#description' => t('When enabled, the RoboTagger-API caches the returned data by the RoboTagger-Webservice.'),
  );
  $nodetypes = node_type_get_names();
  $form['robotagger_global_wrapper']['robotagger_global_nodetypes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Supported content types'),
    '#options' => $nodetypes,
    '#default_value' => variable_get('robotagger_global_nodetypes', array()),
    '#description' => t('Choose the content types for RoboTagger'),
  );
  $languages = _robotagger_get_all_supported_languages();
  $options = array();
  foreach ($languages as $langcode => $langname) {
    $options = _robotagger_get_vocabularies($langcode, 'db');
    if (empty($options)) {
      continue;
    }
    $values = variable_get('robotagger_global_vocs_'.$langcode, array());
    $form['robotagger_global_wrapper']['robotagger_global_vocs_'.$langcode] = array(
      '#type' => 'checkboxes',
      '#title' => t('Vocabularies for the language @language', array('@language' => $langname)),
      '#options' => $options,
      '#default_value' => $values,
    );
    $text = array_filter($values) ? t('Deselect all') : t('Select all');
    $form['robotagger_global_wrapper']['js'.$langcode] = array(
      '#type' => 'item',
      '#markup' => '<a style="cursor: pointer;" class="robotagger-backend-js-select-all">' . $text . '</a>',
    );
  }
  $form['robotagger_nodetype_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content type settings'),
    '#description' => t('Please note. You need to add the right taxonomy fields for each vocabularies in each content type, where you want to use it.'),
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
    '#states' => array(
      'visible' => array("input[name^='robotagger_global_nodetypes']" => array(
        'checked' => TRUE,
      )),
    ),
  );

  foreach ($nodetypes as $mname => $uiname) {
    $w_key = 'robotagger_nodetype_' . $mname . '_wrapper';
    $form['robotagger_nodetype_wrapper'][$w_key] = array(
      '#type' => 'fieldset',
      '#title' => $uiname,
      '#collapsed' => TRUE,
      '#collapsible' => TRUE,
      '#states' => array(
        'visible' => array("input[name='robotagger_global_nodetypes[$mname]']" => array(
          'checked' => TRUE,
         )),
      ),
    );
    $s_key = 'robotagger_nodetype_' . $mname . '_settings';
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_use_global'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use default settings'),
      '#description' => t('All made settings below are ignored and the global setting are used.'),
      '#default_value' => variable_get($s_key.'_use_global', 1),
    );
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_sendrequest'] = array(
      '#type' => 'select',
      '#title' => t('Send request'),
      '#options' => array('auto' => t('automatic'), 'manual' => t('manual')),
      '#default_value' => variable_get($s_key.'_sendrequest', 'auto'),
      '#description' => t('automatic - After submitting the content, request the RoboTagger-Webservice'),
    );
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_confirmtags'] = array(
      '#type' => 'select',
      '#title' => t('Confirm returned tags'),
      '#options' => array('auto' => t('automatic'), 'manual' => t('manual')),
      '#default_value' => variable_get($s_key.'_confirmtags', 'auto'),
      '#description' => t('manual - Before the tags are saved, you can confirm or edit them. automatic - the module save the tags immediately'),
    );
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_caching'] = array(
      '#type' => 'select',
      '#title' => t('Caching returned data'),
      '#options' => array(TRUE => t('yes'), FALSE => t('no')),
      '#default_value' => variable_get($s_key.'_caching', TRUE),
    );
    foreach ($languages as $langcode => $langname) {
      $options = _robotagger_get_vocabularies($langcode, 'db');
      if (empty($options)) {
        continue;
      }
      $values = variable_get($s_key.'_vocs_'.$langcode, array());
      $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_vocs_'.$langcode] = array(
        '#type' => 'checkboxes',
        '#title' => t('Vocabularies for the language @language', array('@language' => $langname)),
        '#options' => $options,
        '#default_value' => $values,
      );
    }
  }
  return system_settings_form($form);
}
