<?php

/**
 * @file robotagger.pages.inc
 */

function robotagger_node_page($node) {
  $output = '';
  $suggestions = _robotagger_get_suggestions($node);
  $declined = _robotagger_get_declined_terms($node);
  if (empty($suggestions) && empty($declined)) {
    $output .= t('There no suggestions to confirm.');
  }
  else{
    $form = drupal_get_form('robotagger_node_page_form', $node, $suggestions, $declined);
    $output .= drupal_render($form);
  }
  $settings = _robotagger_get_settings($node->type, $node->language);
  if ($settings->sendrequest == 'manual') {
    $form = drupal_get_form('robotagger_sendrequest_form', $node);
    $output .= drupal_render($form);
  }
  return $output;
}

function robotagger_node_page_form($form, $form_state, $node, $suggestions, $declined) {
  $form['#tree'] = TRUE;
  $form['#node'] = $node;
  $form['#attached']['js'] = array(drupal_get_path('module', 'robotagger') . '/js/robotagger.js');
  $temp = array();
  foreach ($suggestions as $suggestion) {
    $temp[$suggestion->vid][] = $suggestion->annotation;
    $form['#node']->vids[] = $suggestion->vid;
  }
  $form['vids'][] = array(
    '#type' => 'item',
    '#markup' => '<h2>' . t('Suggestions') . '</h2>',
  );
  foreach ($temp as $vid => $annotations) {
    $voc = taxonomy_vocabulary_load($vid);
    $header = array(
      'title' => array('data' => $voc->name, 'field' => 'n.title'),
    );
    $options = array();
    foreach($annotations as $annotation) {
      $options[$annotation] = array(
        'title' => $annotation,
      );
    }
    $form['vids'][$vid] = array(
      '#type' => 'tableselect',
      '#options' => $options,
      '#header' => $header,
      '#attributes' => array('style' => 'width: 30%; display: inline;'),
    );
  }

  if (!empty($declined)) {
    $form['declined'][] = array(
      '#type' => 'item',
      '#markup' => '<h2>' . t('Declined suggestions') . '</h2>',
    );
    foreach($declined as $dvid => $dannotations) {
      $dvoc = taxonomy_vocabulary_load($dvid);
      $dheader = array(
        'title' => array('data' => $dvoc->name, 'field' => 'n.title'),
      );
      $doptions = array();
      foreach ($dannotations as $dannotation) {
        $doptions[$dannotation] = array(
          'title' => $dannotation,
        );
      }
      $form['declined'][$dvid] = array(
        '#type' => 'tableselect',
        '#options' => $doptions,
        '#header' => $dheader,
        '#attributes' => array('style' => 'width: 30%; display: inline;'),
      );
    }
  }
  $form['action'] = array(
    '#type' => 'select',
    '#options' => array('confirm' => t('Confirm'), 'decline' => t('Decline')),
  );
  $form['button'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    '#attributes' => array('style' => 'display: block;'),
  );
  return $form;
}

function robotagger_node_page_form_submit($form, $form_state) {
  include_once dirname(__FILE__).'/robotagger.process.inc';
  return _robotagger_process_node(FALSE, 'form', array($form, $form_state));
}

function robotagger_sendrequest_form($form, $form_state, $node) {
  $form['#node'] = $node;
  $form['button'] = array(
    '#type' => 'submit',
    '#value' => t('Send Request'),
  );
  return $form;
}

function robotagger_sendrequest_form_submit($form, $form_state) {
  $node = (object)$form['#node'];
  $node->rtcheck_sendrequest = FALSE;
  include_once dirname(__FILE__).'/robotagger.process.inc';
  return _robotagger_process_node($node, 'nodeapi', array($form, $form_state));
}
