<?php

/**
 * @file robotagger.process.inc
 */

function _robotagger_process_node($node = FALSE, $type = '', $args = array()) {
  if ($type == 'nodeapi') {
    return _robotagger_process_node_nodeapi($node);
  }
  elseif ($type == 'form') {
    return _robotagger_process_node_form($args[0], $args[1]);
  }
}

function _robotagger_process_node_nodeapi($node) {
  if (empty($node->language)) {
    return;
  }
  $langcode = $node->language;
  $settings = _robotagger_get_settings($node->type, $langcode);
  if ($settings->sendrequest == 'disabled' || !$settings->isallow) {
    return;
  }
  if ($settings->sendrequest == 'manual' && $node->rtcheck_sendrequest == TRUE) {
    return;
  }
  $vids = array_filter($settings->vids);//remove disabled vids
  if (empty($vids)) {
    return;
  }

  $fields = field_info_fields();
  $vocabs = array();
  foreach ($fields as $field_name => $field) {
    // $field['bundles'] contains names of bundles and entities associated with this field.
    // keys are entity types, values are arrays of bundle names.
    if ($field['type'] == 'taxonomy_term_reference' && !empty($field['bundles']['node']) && in_array($node->type, $field['bundles']['node'])) {
      // Collect all vocabularies allowed for the field.
      foreach ($field['settings']['allowed_values'] as $allowed_values) {
        $vocabs[] = $allowed_values['vocabulary'];
      }
    }
  }

  $query = db_select('taxonomy_vocabulary', 'tv')
  ->fields('tv', array('vid', 'name'))
  ->condition('tv.machine_name', $vocabs, 'IN');
  $results = $query->execute();
  foreach ($results as $result) {
    $nodetypes_vids[$result->vid] = $result->name;
  }

  if (is_array($node->body)) {
    $body = $node->body;
    $text = '';
    foreach ($body as $lang) {
      foreach ($lang as $item) {
        $text .= $item['value'] . ' ';
      }
    }
  }
  else {
    $text = $node->body;
  }
  if (!empty($nodetypes_vids)) {
    $data = robotagger_api_call_webservice($text, $nodetypes_vids, $langcode, $settings->caching);
    if (empty($data)) {
      return;
    }
    watchdog('robotagger', 'Robotagger processing %title.', array('%title' => truncate_utf8($text, 150, TRUE, TRUE)));
    $terms = taxonomy_node_get_terms($node, $key = 'tid');
    $declined = _robotagger_get_declined_terms($node);
    foreach ($nodetypes_vids as $vid => $vocname) {
      if (isset($data[$vocname])) {
        $suggestions = array();
        foreach ($data[$vocname] as &$item) {
          if (($value = trim($item['value'])) && !empty($value)) {
            if (!isset($declined[$vid][$value])) {
              $tid = key(taxonomy_get_term_by_name($value));
              if (!isset($terms[$tid]) || $terms[$tid]->name != $value) {
                $suggestions[$value] = $item;
              }
            }
          }
        }
        if (!empty($suggestions)) {
          if ($settings->confirmtags == 'auto') {
            $node = _robotagger_process_node_autoconfirm($node, $vid, $suggestions);
          }
          elseif ($settings->confirmtags == 'manual') {
            _robotagger_term_draft_save($node, $vid, $suggestions);
          }
        }// end if (!empty($suggestions)) {
      }
    }// end foreach ($nodetypes_vids as $vid => $vocname) {
  }
}

/**
 * Prepared the $node object for taxonomy_node_save().
 *
 * @see robotagger_node_page_form_submit();
 *
 * @param array $form
 * @param array $form_state
 */

function _robotagger_process_node_form($form, $form_state) {
  $node = $form['#node'];
  $terms = array();
  $index = array();
  $results = db_select('taxonomy_index', 'ti')
  ->fields('ti', array('nid', 'tid'))
  ->condition('ti.nid', $node->nid)
  ->execute();
  foreach ($results as $result) {
    $index[] = $result->tid;
  }
  if (isset($form_state['values']['vids'])) {
    foreach($form_state['values']['vids'] as $vid => $annotypes) {
      foreach($annotypes as $annotype => $checked) {
        if (!empty($checked)) {
          $terms[$vid][] = $annotype;
        }
      }
    }
  }
  if ($form_state['input']['action'] == 'confirm') {
    foreach ($terms as $vid => $annotypes) {
      $items = array();
      $field_name = 'rtg_'.taxonomy_vocabulary_load($vid)->machine_name;
      $field = field_read_field($field_name);
      $instance = field_info_instance('node', $field_name, $node->type);
      $i = 0;
      foreach ($annotypes as $annotype) {
        $term = new stdClass;
        $tid = key(taxonomy_get_term_by_name($annotype));
        $term->tid = $tid;
        $term->vid = $vid;
        $term->name= $annotype;
        if (!empty($term->tid)) {
          $status = taxonomy_term_save($term);
          if (!in_array($term->tid, $index)) {
            $items[$i]['tid'] = $term->tid;
            $items[$i]['name']= $annotype;
            $node->{$field_name}[LANGUAGE_NONE][] = array('tid' => $tid);
          }
        }
        else {
          $status = taxonomy_term_save($term);
          $tid = key(taxonomy_get_term_by_name($term->name));
          $term->tid = $tid;
          $items[$i]['tid'] = $term->tid;
          $items[$i]['name']= $annotype;
          $node->{$field_name}[LANGUAGE_NONE][] = array('tid' => $tid);
        }
        $i++;
      }
      taxonomy_field_insert('node', $node, $field, $instance, LANGUAGE_NONE, $items);
    }
    node_save($node);
    _robotagger_term_draft_delete($node, $terms);
    _robotagger_process_node_form_declined($form, $form_state);
  }
  else {
    _robotagger_decline_terms($node, $terms);
  }
}

function _robotagger_process_node_form_declined($form, $form_state) {
  $node = $form['#node'];
  $terms = array();
  $index = array();
  $results = db_select('taxonomy_index', 'ti')
  ->fields('ti', array('nid', 'tid'))
  ->condition('ti.nid', $node->nid)
  ->execute();
  foreach ($results as $result) {
    $index[] = $result->tid;
  }
  if (isset($form_state['values']['declined'])) {
    foreach($form_state['values']['declined'] as $vid => $annotypes) {
      foreach($annotypes as $annotype => $checked) {
        if (!empty($checked)) {
          $terms[$vid][] = $annotype;
        }
      }
    }
  }
  if ($form_state['input']['action'] == 'confirm') {
    foreach ($terms as $vid => $annotypes) {
      $items = array();
      $field_name = 'rtg_'.taxonomy_vocabulary_load($vid)->machine_name;
      $field = field_read_field($field_name);
      $instance = field_info_instance('node', $field_name, $node->type);
      $i = 0;
      foreach ($annotypes as $annotype) {
        $term = new stdClass;
        $tid = key(taxonomy_get_term_by_name($annotype));
        $term->tid = $tid;
        $term->vid = $vid;
        $term->name= $annotype;
        if (!empty($term->tid)) {
          $status = taxonomy_term_save($term);
          if (!in_array($term->tid, $index)) {
            $items[$i]['tid'] = $term->tid;
            $items[$i]['name']= $annotype;
            $node->{$field_name}[LANGUAGE_NONE][] = array('tid' => $tid);
          }
        }
        else {
          $status = taxonomy_term_save($term);
          $tid = key(taxonomy_get_term_by_name($term->name));
          $term->tid = $tid;
          $items[$i]['tid'] = $term->tid;
          $items[$i]['name']= $annotype;
          $node->{$field_name}[LANGUAGE_NONE][] = array('tid' => $tid);
        }
        $i++;
      }
      taxonomy_field_insert('node', $node, $field, $instance, LANGUAGE_NONE, $items);
    }
    node_save($node);
    _robotagger_term_draft_delete($node, $terms);
  }
  else {
    _robotagger_decline_terms($node, $terms);
  }
}

function _robotagger_process_node_autoconfirm($node, $vid, $suggestions) {
  $terms = array();
  $index = array();
  $results = db_select('taxonomy_index', 'ti')
  ->fields('ti', array('nid', 'tid'))
  ->condition('ti.nid', $node->nid)
  ->execute();
  foreach ($results as $result) {
    $index[] = $result->tid;
  }
  foreach ($suggestions as $annotype => $suggestion) {
    $items = array();
    $field_name = 'rtg_'.taxonomy_vocabulary_load($vid)->machine_name;
    $field = field_read_field($field_name);
    $instance = field_info_instance('node', $field_name, $node->type);
    $i = 0;
    $term = new stdClass;
    $tid = key(taxonomy_get_term_by_name($annotype));
    $term->tid = $tid;
    $term->vid = $vid;
    $term->name = $annotype;
    if (!empty($term->tid)) {
      taxonomy_term_save($term);
      if (!in_array($term->tid, $index)) {
        $items[$i]['tid'] = $term->tid;
        $items[$i]['name']= $annotype;
        $node->{$field_name}[LANGUAGE_NONE][] = array('tid' => $tid);
      }
    }
    else {
      taxonomy_term_save($term);
      $tid = key(taxonomy_get_term_by_name($term->name));
      $term->tid = $tid;
      $items[$i]['tid'] = $term->tid;
      $items[$i]['name']= $annotype;
      $node->{$field_name}[LANGUAGE_NONE][] = array('tid' => $tid);
    }
    $i++;
  }
  _robotagger_term_draft_delete($node, $terms);
  return $node;
}

function _robotagger_term_save($node, $vid, $suggestions, $terms) {
  $new_terms = $inserted = array();
  $names = array_keys($suggestions);
  $query = db_select('taxonomy_term_data', 'ttd')
  ->condition('ttd.name', $names, 'IN')
  ->condition('ttd.language', $node->language)
  ->condition('ttd.vid', $vid)
  ->fields('ttd', array('tid', 'name'));
  $results = $query->execute();
  $tids = array();
  foreach ($results as $result) {
    $tids[$results->name] = $result->tid;
  }
  foreach ($suggestions as $suggestion) {
    if (isset($tids[$suggestion['value']])) {
      $tid =  $tids[$suggestion['value']];
      $new_terms[$tid] = $tid;
    }
    else{
      $edit = array(
        'vid' => $vid,
        'name' => $suggestion['value'],
      );
      $status = taxonomy_term_save($edit);
      if (isset($edit['tid']) && $edit['tid']) {
        $new_terms[$edit['tid']] = $edit['tid'];
      }
    }
  }
  if (!empty($new_terms)) {
    foreach ($new_terms as $tid) {
      if (!isset($inserted[$tid]) && !isset($terms[$tid])) {
        db_insert('taxonomy_index')
        ->fields(array(
            'nid' => $node->nid,
            'tid' => $tid,
            'sticky' => '0',
            'created' => time()))
        ->execute();
        $inserted[$tid] = TRUE;
      }
    }
  }
}

/**
 * Saves the terms in the robotagger db table, so that the user can confirm the tags.
 *
 * @param object $node
 * @param vid $vid
 * @param array $suggestions
 */
function _robotagger_term_draft_save($node, $vid, $suggestions) {
  if (isset($node->nid)) {
    $oldsuggestions = _robotagger_get_suggestions($node, $vid);
    $new_terms = array();
    foreach ($suggestions as $annotation => $suggestion) {
      if (!isset($oldsuggestions[$suggestion['value']])) {
        $new_terms[$suggestion['value']] = $suggestion;
      }
    }
    if (!empty($new_terms)) {
      $names = array_keys($new_terms);
      $query = db_select('taxonomy_term_data', 'ttd')
      ->fields('ttd', array('tid', 'name'))
      ->condition('ttd.name', $names, 'IN')
      ->condition('ttd.language')
      ->condition('ttd.vid');
      $results = $query->execute();
      $tids = array();
      foreach ($results as $result) {
        $tids[$result->name] = $result->tid;
      }
      $new = FALSE;
      foreach ($new_terms as $new_term) {
        if (!isset($tids[$new_term['value']])) {
          db_insert('robotagger')
          ->fields(array(
              'nid' => $node->nid,
              'nvid' => $node->vid,
              'vid' => $vid,
              'annotation' => $new_term['value'],
              'occurences' => $new_term['occurences'],
              'threshold' => $new_term['threshold']))
          ->execute();
          $new = TRUE;
        }
      }
      if ($new) {
        $link = l(t('new tags'), 'node/' . $node->nid . '/robotagger');
        drupal_set_message(t('The RoboTagger-Webservice was requested. Please confirm the returned !new_tags', array('!new_tags' => $link)), 'status', FALSE);
      }
    }
  }
}

/**
 * Removes/Updates terms from the robotagger db table.
 *
 * @param object $node
 * @param array $suggestions
 */
function _robotagger_term_draft_delete($node, $suggestions) {
  if (!empty($suggestions) && !empty($node->nid)) {
    foreach ($suggestions as $vid => $names) {
      foreach ($names as $name) {
        db_delete('robotagger')
        ->condition('nid', $node->nid)
        ->condition('nvid', $node->vid)
        ->condition('vid', $vid)
        ->condition('annotation', $name)
        ->execute();
      }
    }
  }
}

function _robotagger_decline_terms($node, $suggestions) {
  if (!empty($suggestions) && !empty($node->nid)) {
    foreach ($suggestions as $vid => $names) {
      foreach ($names as $name) {
        db_update('robotagger')
        ->fields(array('declined' => 1))
        ->condition('nid', $node->nid)
        ->condition('nvid', $node->vid)
        ->condition('vid', $vid)
        ->condition('annotation', $name)
        ->execute();
      }
    }
  }
}

/**
 * New function (Drupal 7)
 * Implements the old function taxonomy_node_get_terms (Drupal 6)
 *
 */
function taxonomy_node_get_terms($node, $key = 'tid') {
  static $terms;
  if (!isset($terms[$node->vid][$key])) {
    $query = db_select('taxonomy_index', 'r');
    $t_alias = $query->join('taxonomy_term_data', 't', 'r.tid = t.tid');
    $v_alias = $query->join('taxonomy_vocabulary', 'v', 't.vid = v.vid');
    $query->fields( $t_alias );
    $query->condition("r.nid", $node->nid);
    $results = $query->execute();
    $terms[$node->vid][$key] = array();
    foreach ($results as $term) {
      $terms[$node->vid][$key][$term->$key] = $term;
    }
  }
  return $terms[$node->vid][$key];
}
/** End new function */